@@ -1,4 +1,4 @@
-// Copyright 2015-2017 Benjamin Fry <benjaminfry@me.com>
+// Copyright 2015-2023 Benjamin Fry <benjaminfry@me.com>
 //
 // Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
 // http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
@@ -7,38 +7,46 @@
 
 //! Caching related functionality for the Resolver.
 
-use std::borrow::Cow;
-use std::error::Error;
-use std::net::{Ipv4Addr, Ipv6Addr};
-use std::pin::Pin;
-use std::sync::atomic::{AtomicU8, Ordering};
-use std::sync::Arc;
-use std::time::Instant;
+use std::{
+    borrow::Cow,
+    error::Error,
+    pin::Pin,
+    sync::{
+        atomic::{AtomicU8, Ordering},
+        Arc,
+    },
+    time::Instant,
+};
 
 use futures_util::future::Future;
+use once_cell::sync::Lazy;
 
-use proto::error::ProtoError;
-use proto::op::{Query, ResponseCode};
-use proto::rr::domain::usage::{
-    ResolverUsage, DEFAULT, INVALID, IN_ADDR_ARPA_127, IP6_ARPA_1, LOCAL,
-    LOCALHOST as LOCALHOST_usage, ONION,
+use crate::{
+    dns_lru::{self, DnsLru, TtlConfig},
+    error::{ResolveError, ResolveErrorKind},
+    lookup::Lookup,
+    proto::{
+        error::ProtoError,
+        op::{Query, ResponseCode},
+        rr::{
+            domain::usage::{
+                ResolverUsage, DEFAULT, INVALID, IN_ADDR_ARPA_127, IP6_ARPA_1, LOCAL,
+                LOCALHOST as LOCALHOST_usage, ONION,
+            },
+            rdata::{A, AAAA, CNAME, PTR, SOA},
+            resource::RecordRef,
+            DNSClass, Name, RData, Record, RecordType,
+        },
+        xfer::{DnsHandle, DnsRequestOptions, DnsResponse, FirstAnswer},
+    },
 };
-use proto::rr::rdata::SOA;
-use proto::rr::{DNSClass, Name, RData, Record, RecordType};
-use proto::xfer::{DnsHandle, DnsRequestOptions, DnsResponse, FirstAnswer};
-
-use crate::dns_lru::DnsLru;
-use crate::dns_lru::{self, TtlConfig};
-use crate::error::*;
-use crate::lookup::Lookup;
 
 const MAX_QUERY_DEPTH: u8 = 8; // arbitrarily chosen number...
 
-lazy_static! {
-    static ref LOCALHOST: RData = RData::PTR(Name::from_ascii("localhost.").unwrap());
-    static ref LOCALHOST_V4: RData = RData::A(Ipv4Addr::new(127, 0, 0, 1));
-    static ref LOCALHOST_V6: RData = RData::AAAA(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1));
-}
+static LOCALHOST: Lazy<RData> =
+    Lazy::new(|| RData::PTR(PTR(Name::from_ascii("localhost.").unwrap())));
+static LOCALHOST_V4: Lazy<RData> = Lazy::new(|| RData::A(A::new(127, 0, 0, 1)));
+static LOCALHOST_V6: Lazy<RData> = Lazy::new(|| RData::AAAA(AAAA::new(0, 0, 0, 0, 0, 0, 0, 1)));
 
 struct DepthTracker {
     query_depth: Arc<AtomicU8>,
@@ -172,7 +180,7 @@ where
         let is_dnssec = client.client.is_verifying_dnssec();
 
         // first transition any polling that is needed (mutable refs...)
-        if let Some(cached_lookup) = client.from_cache(&query) {
+        if let Some(cached_lookup) = client.lookup_from_cache(&query) {
             return cached_lookup;
         };
 
@@ -192,7 +200,7 @@ where
         };
 
         // TODO: take all records and cache them?
-        //  if it's DNSSec they must be signed, otherwise?
+        //  if it's DNSSEC they must be signed, otherwise?
         let records: Result<Records, ResolveError> = match response_message {
             // this is the only cacheable form
             Err(ResolveError {
@@ -244,7 +252,7 @@ where
     }
 
     /// Check if this query is already cached
-    fn from_cache(&self, query: &Query) -> Option<Result<Lookup, ResolveError>> {
+    fn lookup_from_cache(&self, query: &Query) -> Option<Result<Lookup, ResolveError>> {
         self.lru.get(query, Instant::now())
     }
 
@@ -254,7 +262,7 @@ where
     ///  and a record for the name, regardless of CNAME presence, what have you
     ///  ultimately does not exist.
     ///
-    /// This also handles empty responses in the same way. When performing DNSSec enabled queries, we should
+    /// This also handles empty responses in the same way. When performing DNSSEC enabled queries, we should
     ///  never enter here, and should never cache unless verified requests.
     ///
     /// TODO: should this should be expanded to do a forward lookup? Today, this will fail even if there are
@@ -263,13 +271,13 @@ where
     /// # Arguments
     ///
     /// * `message` - message to extract SOA, etc, from for caching failed requests
-    /// * `valid_nsec` - species that in DNSSec mode, this request is safe to cache
+    /// * `valid_nsec` - species that in DNSSEC mode, this request is safe to cache
     /// * `negative_ttl` - this should be the SOA minimum for negative ttl
     fn handle_nxdomain(
         is_dnssec: bool,
         valid_nsec: bool,
         query: Query,
-        soa: Option<SOA>,
+        soa: Option<Record<SOA>>,
         negative_ttl: Option<u32>,
         response_code: ResponseCode,
         trusted: bool,
@@ -303,14 +311,14 @@ where
         options: DnsRequestOptions,
         is_dnssec: bool,
         query: &Query,
-        mut response: DnsResponse,
+        response: DnsResponse,
         mut preserved_records: Vec<(Record, u32)>,
     ) -> Result<Records, ResolveError> {
         // initial ttl is what CNAMES for min usage
         const INITIAL_TTL: u32 = dns_lru::MAX_TTL;
 
         // need to capture these before the subsequent and destructive record processing
-        let soa = response.soa();
+        let soa = response.soa().as_ref().map(RecordRef::to_owned);
         let negative_ttl = response.negative_ttl();
         let response_code = response.response_code();
 
@@ -334,10 +342,10 @@ where
                         (Cow::Borrowed(query.name()), INITIAL_TTL, false),
                         |(search_name, cname_ttl, was_cname), r| {
                             match r.data() {
-                                Some(RData::CNAME(ref cname)) => {
+                                Some(RData::CNAME(CNAME(ref cname))) => {
                                     // take the minimum TTL of the cname_ttl and the next record in the chain
                                     let ttl = cname_ttl.min(r.ttl());
-                                    debug_assert_eq!(r.rr_type(), RecordType::CNAME);
+                                    debug_assert_eq!(r.record_type(), RecordType::CNAME);
                                     if search_name.as_ref() == r.name() {
                                         return (Cow::Owned(cname.clone()), ttl, true);
                                     }
@@ -345,7 +353,7 @@ where
                                 Some(RData::SRV(ref srv)) => {
                                     // take the minimum TTL of the cname_ttl and the next record in the chain
                                     let ttl = cname_ttl.min(r.ttl());
-                                    debug_assert_eq!(r.rr_type(), RecordType::SRV);
+                                    debug_assert_eq!(r.record_type(), RecordType::SRV);
 
                                     // the search name becomes the srv.target
                                     return (Cow::Owned(srv.target().clone()), ttl, true);
@@ -359,6 +367,7 @@ where
                 };
 
             // take all answers. // TODO: following CNAMES?
+            let mut response = response.into_message();
             let answers = response.take_answers();
             let additionals = response.take_additionals();
             let name_servers = response.take_name_servers();
@@ -381,29 +390,25 @@ where
                     if query.query_class() == r.dns_class() {
                         // standard evaluation, it's an any type or it's the requested type and the search_name matches
                         #[allow(clippy::suspicious_operation_groupings)]
-                        if (query.query_type().is_any() || query.query_type() == r.rr_type())
+                        if (query.query_type().is_any() || query.query_type() == r.record_type())
                             && (search_name.as_ref() == r.name() || query.name() == r.name())
                         {
                             found_name = true;
                             return Some((r, ttl));
                         }
-                        // CNAME evaluation, it's an A/AAAA lookup and the record is from the CNAME lookup chain.
-                        if client.preserve_intermediates
-                            && r.rr_type() == RecordType::CNAME
-                            && (query.query_type() == RecordType::A
-                                || query.query_type() == RecordType::AAAA)
-                        {
+                        // CNAME evaluation, the record is from the CNAME lookup chain.
+                        if client.preserve_intermediates && r.record_type() == RecordType::CNAME {
                             return Some((r, ttl));
                         }
                         // srv evaluation, it's an srv lookup and the srv_search_name/target matches this name
                         //    and it's an IP
                         if query.query_type().is_srv()
-                            && r.rr_type().is_ip_addr()
+                            && r.record_type().is_ip_addr()
                             && search_name.as_ref() == r.name()
                         {
                             found_name = true;
                             Some((r, ttl))
-                        } else if query.query_type().is_ns() && r.rr_type().is_ip_addr() {
+                        } else if query.query_type().is_ns() && r.record_type().is_ip_addr() {
                             Some((r, ttl))
                         } else {
                             None
@@ -444,7 +449,7 @@ where
             })
         } else {
             // TODO: review See https://tools.ietf.org/html/rfc2308 for NoData section
-            // Note on DNSSec, in secure_client_handle, if verify_nsec fails then the request fails.
+            // Note on DNSSEC, in secure_client_handle, if verify_nsec fails then the request fails.
             //   this will mean that no unverified negative caches will make it to this point and be stored
             Err(Self::handle_nxdomain(
                 is_dnssec,
@@ -477,7 +482,7 @@ where
     }
 
     /// Flushes/Removes all entries from the cache
-    pub fn clear_cache(&mut self) {
+    pub fn clear_cache(&self) {
         self.lru.clear();
     }
 }
@@ -503,6 +508,7 @@ mod tests {
     use proto::op::{Message, Query};
     use proto::rr::rdata::SRV;
     use proto::rr::{Name, Record};
+    use trust_dns_proto::rr::rdata::NS;
 
     use super::*;
     use crate::lookup_ip::tests::*;
@@ -543,7 +549,7 @@ mod tests {
                 Record::from_rdata(
                     query.name().clone(),
                     u32::max_value(),
-                    RData::A(Ipv4Addr::new(127, 0, 0, 1)),
+                    RData::A(A::new(127, 0, 0, 1)),
                 ),
                 u32::max_value(),
             )],
@@ -563,7 +569,7 @@ mod tests {
 
         assert_eq!(
             ips.iter().cloned().collect::<Vec<_>>(),
-            vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))]
+            vec![RData::A(A::new(127, 0, 0, 1))]
         );
     }
 
@@ -584,7 +590,7 @@ mod tests {
 
         assert_eq!(
             ips.iter().cloned().collect::<Vec<_>>(),
-            vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))]
+            vec![RData::A(A::new(127, 0, 0, 1))]
         );
 
         // next should come from cache...
@@ -601,7 +607,7 @@ mod tests {
 
         assert_eq!(
             ips.iter().cloned().collect::<Vec<_>>(),
-            vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))]
+            vec![RData::A(A::new(127, 0, 0, 1))]
         );
     }
 
@@ -615,9 +621,9 @@ mod tests {
         message.insert_answers(vec![Record::from_rdata(
             Name::from_str("www.example.com.").unwrap(),
             86400,
-            RData::CNAME(Name::from_str("actual.example.com.").unwrap()),
+            RData::CNAME(CNAME(Name::from_str("actual.example.com.").unwrap())),
         )]);
-        Ok(message.into())
+        Ok(DnsResponse::from_message(message).unwrap())
     }
 
     #[allow(clippy::unnecessary_wraps)]
@@ -637,7 +643,7 @@ mod tests {
                 Name::from_str("www.example.com.").unwrap(),
             )),
         )]);
-        Ok(message.into())
+        Ok(DnsResponse::from_message(message).unwrap())
     }
 
     #[allow(clippy::unnecessary_wraps)]
@@ -650,9 +656,9 @@ mod tests {
         message.insert_answers(vec![Record::from_rdata(
             Name::from_str("www.example.com.").unwrap(),
             86400,
-            RData::NS(Name::from_str("www.example.com.").unwrap()),
+            RData::NS(NS(Name::from_str("www.example.com.").unwrap())),
         )]);
-        Ok(message.into())
+        Ok(DnsResponse::from_message(message).unwrap())
     }
 
     fn no_recursion_on_query_test(query_type: RecordType) {
@@ -672,7 +678,9 @@ mod tests {
 
         assert_eq!(
             ips.iter().cloned().collect::<Vec<_>>(),
-            vec![RData::CNAME(Name::from_str("actual.example.com.").unwrap())]
+            vec![RData::CNAME(CNAME(
+                Name::from_str("actual.example.com.").unwrap()
+            ))]
         );
     }
 
@@ -720,26 +728,29 @@ mod tests {
     fn test_single_srv_query_response() {
         let cache = DnsLru::new(1, dns_lru::TtlConfig::default());
 
-        let mut message = srv_message().unwrap();
+        let mut message = srv_message().unwrap().into_message();
         message.add_answer(Record::from_rdata(
             Name::from_str("www.example.com.").unwrap(),
             86400,
-            RData::CNAME(Name::from_str("actual.example.com.").unwrap()),
+            RData::CNAME(CNAME(Name::from_str("actual.example.com.").unwrap())),
         ));
         message.insert_additionals(vec![
             Record::from_rdata(
                 Name::from_str("actual.example.com.").unwrap(),
                 86400,
-                RData::A(Ipv4Addr::new(127, 0, 0, 1)),
+                RData::A(A::new(127, 0, 0, 1)),
             ),
             Record::from_rdata(
                 Name::from_str("actual.example.com.").unwrap(),
                 86400,
-                RData::AAAA(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
+                RData::AAAA(AAAA::new(0, 0, 0, 0, 0, 0, 0, 1)),
             ),
         ]);
 
-        let client = mock(vec![error(), Ok(message)]);
+        let client = mock(vec![
+            error(),
+            Ok(DnsResponse::from_message(message).unwrap()),
+        ]);
         let client = CachingClient::with_cache(cache, client, false);
 
         let ips = block_on(CachingClient::inner_lookup(
@@ -762,8 +773,8 @@ mod tests {
                     443,
                     Name::from_str("www.example.com.").unwrap(),
                 )),
-                RData::A(Ipv4Addr::new(127, 0, 0, 1)),
-                RData::AAAA(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
+                RData::A(A::new(127, 0, 0, 1)),
+                RData::AAAA(AAAA::new(0, 0, 0, 0, 0, 0, 0, 1)),
             ]
         );
     }
@@ -789,7 +800,7 @@ mod tests {
     //         ),
     //     ]);
 
-    //     let mut client = mock(vec![error(), Ok(message.into()), srv_message()]);
+    //     let mut client = mock(vec![error(), Ok(DnsResponse::from_message(message).unwrap()), srv_message()]);
 
     //     let ips = QueryState::lookup(
     //         Query::query(
@@ -821,26 +832,29 @@ mod tests {
     fn test_single_ns_query_response() {
         let cache = DnsLru::new(1, dns_lru::TtlConfig::default());
 
-        let mut message = ns_message().unwrap();
+        let mut message = ns_message().unwrap().into_message();
         message.add_answer(Record::from_rdata(
             Name::from_str("www.example.com.").unwrap(),
             86400,
-            RData::CNAME(Name::from_str("actual.example.com.").unwrap()),
+            RData::CNAME(CNAME(Name::from_str("actual.example.com.").unwrap())),
         ));
         message.insert_additionals(vec![
             Record::from_rdata(
                 Name::from_str("actual.example.com.").unwrap(),
                 86400,
-                RData::A(Ipv4Addr::new(127, 0, 0, 1)),
+                RData::A(A::new(127, 0, 0, 1)),
             ),
             Record::from_rdata(
                 Name::from_str("actual.example.com.").unwrap(),
                 86400,
-                RData::AAAA(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
+                RData::AAAA(AAAA::new(0, 0, 0, 0, 0, 0, 0, 1)),
             ),
         ]);
 
-        let client = mock(vec![error(), Ok(message)]);
+        let client = mock(vec![
+            error(),
+            Ok(DnsResponse::from_message(message).unwrap()),
+        ]);
         let client = CachingClient::with_cache(cache, client, false);
 
         let ips = block_on(CachingClient::inner_lookup(
@@ -854,9 +868,9 @@ mod tests {
         assert_eq!(
             ips.iter().cloned().collect::<Vec<_>>(),
             vec![
-                RData::NS(Name::from_str("www.example.com.").unwrap()),
-                RData::A(Ipv4Addr::new(127, 0, 0, 1)),
-                RData::AAAA(Ipv6Addr::new(0, 0, 0, 0, 0, 0, 0, 1)),
+                RData::NS(NS(Name::from_str("www.example.com.").unwrap())),
+                RData::A(A::new(127, 0, 0, 1)),
+                RData::AAAA(AAAA::new(0, 0, 0, 0, 0, 0, 0, 1)),
             ]
         );
     }
@@ -870,12 +884,12 @@ mod tests {
         message.insert_answers(vec![Record::from_rdata(
             Name::from_str("ttl.example.com.").unwrap(),
             first,
-            RData::CNAME(Name::from_str("actual.example.com.").unwrap()),
+            RData::CNAME(CNAME(Name::from_str("actual.example.com.").unwrap())),
         )]);
         message.insert_additionals(vec![Record::from_rdata(
             Name::from_str("actual.example.com.").unwrap(),
             second,
-            RData::A(Ipv4Addr::new(127, 0, 0, 1)),
+            RData::A(A::new(127, 0, 0, 1)),
         )]);
 
         let records = CachingClient::handle_noerror(
@@ -883,7 +897,7 @@ mod tests {
             DnsRequestOptions::default(),
             false,
             &Query::query(Name::from_str("ttl.example.com.").unwrap(), RecordType::A),
-            message.into(),
+            DnsResponse::from_message(message).unwrap(),
             vec![],
         );
 
@@ -1004,7 +1018,7 @@ mod tests {
     fn test_no_error_on_dot_local_no_mdns() {
         let cache = DnsLru::new(1, dns_lru::TtlConfig::default());
 
-        let mut message = srv_message().unwrap();
+        let mut message = srv_message().unwrap().into_message();
         message.add_query(Query::query(
             Name::from_ascii("www.example.local.").unwrap(),
             RecordType::A,
@@ -1012,10 +1026,13 @@ mod tests {
         message.add_answer(Record::from_rdata(
             Name::from_str("www.example.local.").unwrap(),
             86400,
-            RData::A(Ipv4Addr::new(127, 0, 0, 1)),
+            RData::A(A::new(127, 0, 0, 1)),
         ));
 
-        let client = mock(vec![error(), Ok(message)]);
+        let client = mock(vec![
+            error(),
+            Ok(DnsResponse::from_message(message).unwrap()),
+        ]);
         let mut client = CachingClient::with_cache(cache, client, false);
 
         assert!(block_on(client.lookup(
