@@ -6,6 +6,8 @@
 // copied, modified, or distributed except according to those terms.
 
 //! Configuration for a resolver
+#![allow(clippy::use_self)]
+
 use std::fmt;
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr, SocketAddr};
 use std::ops::{Deref, DerefMut};
@@ -63,6 +65,38 @@ impl ResolverConfig {
         }
     }
 
+    /// Creates a default configuration, using `8.8.8.8`, `8.8.4.4` and `2001:4860:4860::8888`, `2001:4860:4860::8844` (thank you, Google). This limits the registered connections to just TLS lookups
+    ///
+    /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
+    ///
+    /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
+    #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
+    pub fn google_tls() -> Self {
+        Self {
+            // TODO: this should get the hostname and use the basename as the default
+            domain: None,
+            search: vec![],
+            name_servers: NameServerConfigGroup::google_tls(),
+        }
+    }
+
+    /// Creates a default configuration, using `8.8.8.8`, `8.8.4.4` and `2001:4860:4860::8888`, `2001:4860:4860::8844` (thank you, Google). This limits the registered connections to just HTTPS lookups
+    ///
+    /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
+    ///
+    /// NameServerConfigGroups can be combined to use a set of different providers, see `NameServerConfigGroup` and `ResolverConfig::from_parts`
+    #[cfg(feature = "dns-over-https")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
+    pub fn google_https() -> Self {
+        Self {
+            // TODO: this should get the hostname and use the basename as the default
+            domain: None,
+            search: vec![],
+            name_servers: NameServerConfigGroup::google_https(),
+        }
+    }
+
     /// Creates a default configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare).
     ///
     /// Please see: <https://www.cloudflare.com/dns/>
@@ -228,7 +262,7 @@ impl ResolverConfig {
     /// use webpki_roots;
     ///
     /// let mut root_store = RootCertStore::empty();
-    /// root_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(|ta| {
+    /// root_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.iter().map(|ta| {
     ///     OwnedTrustAnchor::from_subject_spki_name_constraints(
     ///         ta.subject,
     ///         ta.spki,
@@ -284,6 +318,10 @@ pub enum Protocol {
     #[cfg(feature = "dns-over-https")]
     #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     Https,
+    /// QUIC for DNS over QUIC
+    #[cfg(feature = "dns-over-quic")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-quic")))]
+    Quic,
     /// mDNS protocol for performing multicast lookups
     #[cfg(feature = "mdns")]
     #[cfg_attr(docsrs, doc(cfg(feature = "mdns")))]
@@ -293,14 +331,16 @@ pub enum Protocol {
 impl fmt::Display for Protocol {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let protocol = match self {
-            Protocol::Udp => "udp",
-            Protocol::Tcp => "tcp",
+            Self::Udp => "udp",
+            Self::Tcp => "tcp",
             #[cfg(feature = "dns-over-tls")]
-            Protocol::Tls => "tls",
+            Self::Tls => "tls",
             #[cfg(feature = "dns-over-https")]
-            Protocol::Https => "https",
+            Self::Https => "https",
+            #[cfg(feature = "dns-over-quic")]
+            Self::Quic => "quic",
             #[cfg(feature = "mdns")]
-            Protocol::Mdns => "mdns",
+            Self::Mdns => "mdns",
         };
 
         f.write_str(protocol)
@@ -311,14 +351,17 @@ impl Protocol {
     /// Returns true if this is a datagram oriented protocol, e.g. UDP
     pub fn is_datagram(self) -> bool {
         match self {
-            Protocol::Udp => true,
-            Protocol::Tcp => false,
+            Self::Udp => true,
+            Self::Tcp => false,
             #[cfg(feature = "dns-over-tls")]
-            Protocol::Tls => false,
+            Self::Tls => false,
             #[cfg(feature = "dns-over-https")]
-            Protocol::Https => false,
+            Self::Https => false,
+            // TODO: if you squint, this is true...
+            #[cfg(feature = "dns-over-quic")]
+            Self::Quic => true,
             #[cfg(feature = "mdns")]
-            Protocol::Mdns => true,
+            Self::Mdns => true,
         }
     }
 
@@ -330,14 +373,16 @@ impl Protocol {
     /// Is this an encrypted protocol, i.e. TLS or HTTPS
     pub fn is_encrypted(self) -> bool {
         match self {
-            Protocol::Udp => false,
-            Protocol::Tcp => false,
+            Self::Udp => false,
+            Self::Tcp => false,
             #[cfg(feature = "dns-over-tls")]
-            Protocol::Tls => true,
+            Self::Tls => true,
             #[cfg(feature = "dns-over-https")]
-            Protocol::Https => true,
+            Self::Https => true,
+            #[cfg(feature = "dns-over-quic")]
+            Self::Quic => true,
             #[cfg(feature = "mdns")]
-            Protocol::Mdns => false,
+            Self::Mdns => false,
         }
     }
 }
@@ -387,16 +432,18 @@ pub struct NameServerConfig {
     pub tls_dns_name: Option<String>,
     /// Whether to trust `NXDOMAIN` responses from upstream nameservers.
     ///
-    /// When this is `true`, and an empty `NXDOMAIN` response is received, the
-    /// query will not be retried against other configured name servers.
+    /// When this is `true`, and an empty `NXDOMAIN` response or `NOERROR`
+    /// with an empty answers set is received, the
+    /// query will not be retried against other configured name servers if
+    /// the response has the Authoritative flag set.
     ///
-    /// (On an empty `NoError` response, or a response with any other error
+    /// (On a response with any other error
     /// response code, the query will still be retried regardless of this
     /// configuration setting.)
     ///
     /// Defaults to false.
     #[cfg_attr(feature = "serde-config", serde(default))]
-    pub trust_nx_responses: bool,
+    pub trust_negative_responses: bool,
     #[cfg(feature = "dns-over-rustls")]
     #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-rustls")))]
     #[cfg_attr(feature = "serde-config", serde(skip))]
@@ -406,12 +453,27 @@ pub struct NameServerConfig {
     pub bind_addr: Option<SocketAddr>,
 }
 
+impl NameServerConfig {
+    /// Constructs a Nameserver configuration with some basic defaults
+    pub fn new(socket_addr: SocketAddr, protocol: Protocol) -> Self {
+        Self {
+            socket_addr,
+            protocol,
+            trust_negative_responses: true,
+            tls_dns_name: None,
+            #[cfg(feature = "dns-over-rustls")]
+            tls_config: None,
+            bind_addr: None,
+        }
+    }
+}
+
 impl fmt::Display for NameServerConfig {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "{}:", self.protocol)?;
 
         if let Some(ref tls_dns_name) = self.tls_dns_name {
-            write!(f, "{}@", tls_dns_name)?;
+            write!(f, "{tls_dns_name}@")?;
         }
 
         write!(f, "{}", self.socket_addr)
@@ -466,10 +528,15 @@ impl NameServerConfigGroup {
         )
     }
 
+    /// Returns the inner vec of configs
+    pub fn into_inner(self) -> Vec<NameServerConfig> {
+        self.0
+    }
+
     /// Configure a NameServer address and port
     ///
     /// This will create UDP and TCP connections, using the same port.
-    pub fn from_ips_clear(ips: &[IpAddr], port: u16, trust_nx_responses: bool) -> Self {
+    pub fn from_ips_clear(ips: &[IpAddr], port: u16, trust_negative_responses: bool) -> Self {
         let mut name_servers = Self::with_capacity(ips.len());
 
         for ip in ips {
@@ -477,7 +544,7 @@ impl NameServerConfigGroup {
                 socket_addr: SocketAddr::new(*ip, port),
                 protocol: Protocol::Udp,
                 tls_dns_name: None,
-                trust_nx_responses,
+                trust_negative_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
                 bind_addr: None,
@@ -486,7 +553,7 @@ impl NameServerConfigGroup {
                 socket_addr: SocketAddr::new(*ip, port),
                 protocol: Protocol::Tcp,
                 tls_dns_name: None,
-                trust_nx_responses,
+                trust_negative_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
                 bind_addr: None,
@@ -505,7 +572,7 @@ impl NameServerConfigGroup {
         port: u16,
         tls_dns_name: String,
         protocol: Protocol,
-        trust_nx_responses: bool,
+        trust_negative_responses: bool,
     ) -> Self {
         assert!(protocol.is_encrypted());
 
@@ -516,7 +583,7 @@ impl NameServerConfigGroup {
                 socket_addr: SocketAddr::new(*ip, port),
                 protocol,
                 tls_dns_name: Some(tls_dns_name.clone()),
-                trust_nx_responses,
+                trust_negative_responses,
                 #[cfg(feature = "dns-over-rustls")]
                 tls_config: None,
                 bind_addr: None,
@@ -537,9 +604,15 @@ impl NameServerConfigGroup {
         ips: &[IpAddr],
         port: u16,
         tls_dns_name: String,
-        trust_nx_responses: bool,
+        trust_negative_responses: bool,
     ) -> Self {
-        Self::from_ips_encrypted(ips, port, tls_dns_name, Protocol::Tls, trust_nx_responses)
+        Self::from_ips_encrypted(
+            ips,
+            port,
+            tls_dns_name,
+            Protocol::Tls,
+            trust_negative_responses,
+        )
     }
 
     /// Configure a NameServer address and port for DNS-over-HTTPS
@@ -551,9 +624,35 @@ impl NameServerConfigGroup {
         ips: &[IpAddr],
         port: u16,
         tls_dns_name: String,
-        trust_nx_responses: bool,
+        trust_negative_responses: bool,
     ) -> Self {
-        Self::from_ips_encrypted(ips, port, tls_dns_name, Protocol::Https, trust_nx_responses)
+        Self::from_ips_encrypted(
+            ips,
+            port,
+            tls_dns_name,
+            Protocol::Https,
+            trust_negative_responses,
+        )
+    }
+
+    /// Configure a NameServer address and port for DNS-over-QUIC
+    ///
+    /// This will create a QUIC connections.
+    #[cfg(feature = "dns-over-quic")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-quic")))]
+    pub fn from_ips_quic(
+        ips: &[IpAddr],
+        port: u16,
+        tls_dns_name: String,
+        trust_negative_responses: bool,
+    ) -> Self {
+        Self::from_ips_encrypted(
+            ips,
+            port,
+            tls_dns_name,
+            Protocol::Quic,
+            trust_negative_responses,
+        )
     }
 
     /// Creates a default configuration, using `8.8.8.8`, `8.8.4.4` and `2001:4860:4860::8888`, `2001:4860:4860::8844` (thank you, Google).
@@ -563,13 +662,22 @@ impl NameServerConfigGroup {
         Self::from_ips_clear(GOOGLE_IPS, 53, true)
     }
 
+    /// Creates a default configuration, using `8.8.8.8`, `8.8.4.4` and `2001:4860:4860::8888`, `2001:4860:4860::8844` (thank you, Google). This limits the registered connections to just TLS lookups
+    ///
+    /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
+    #[cfg(feature = "dns-over-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-tls")))]
+    pub fn google_tls() -> Self {
+        Self::from_ips_tls(GOOGLE_IPS, 853, "dns.google".to_string(), true)
+    }
+
     /// Creates a default configuration, using `8.8.8.8`, `8.8.4.4` and `2001:4860:4860::8888`, `2001:4860:4860::8844` (thank you, Google). This limits the registered connections to just HTTPS lookups
     ///
     /// Please see Google's [privacy statement](https://developers.google.com/speed/public-dns/privacy) for important information about what they track, many ISP's track similar information in DNS. To use the system configuration see: `Resolver::from_system_conf` and `AsyncResolver::from_system_conf`
     #[cfg(feature = "dns-over-https")]
     #[cfg_attr(docsrs, doc(cfg(feature = "dns-over-https")))]
     pub fn google_https() -> Self {
-        Self::from_ips_https(GOOGLE_IPS, 53, "dns.google".to_string(), true)
+        Self::from_ips_https(GOOGLE_IPS, 443, "dns.google".to_string(), true)
     }
 
     /// Creates a default configuration, using `1.1.1.1`, `1.0.0.1` and `2606:4700:4700::1111`, `2606:4700:4700::1001` (thank you, Cloudflare).
@@ -718,6 +826,25 @@ impl Default for LookupIpStrategy {
     }
 }
 
+/// The strategy for establishing the query order of name servers in a pool.
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[cfg_attr(feature = "serde-config", derive(Serialize, Deserialize))]
+pub enum ServerOrderingStrategy {
+    /// Servers are ordered based on collected query statistics. The ordering
+    /// may vary over time.
+    QueryStatistics,
+    /// The order provided to the resolver is used. The ordering does not vary
+    /// over time.
+    UserProvidedOrder,
+}
+
+impl Default for ServerOrderingStrategy {
+    /// Returns [`ServerOrderingStrategy::QueryStatistics`] as the default.
+    fn default() -> Self {
+        Self::QueryStatistics
+    }
+}
+
 /// Configuration for the Resolver
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
 #[cfg_attr(
@@ -729,9 +856,8 @@ impl Default for LookupIpStrategy {
 #[non_exhaustive]
 pub struct ResolverOpts {
     /// Sets the number of dots that must appear (unless it's a final dot representing the root)
-    ///  that must appear before a query is assumed to include the TLD. The default is one, which
-    ///  means that `www` would never be assumed to be a TLD, and would always be appended to either
-    ///  the search
+    ///  before a query is assumed to include the TLD. The default is one, which means that `www`
+    ///  would never be assumed to be a TLD, and would always be appended to either the search
     pub ndots: usize,
     /// Specify the timeout for a request. Defaults to 5 seconds
     pub timeout: Duration,
@@ -744,7 +870,7 @@ pub struct ResolverOpts {
     pub check_names: bool,
     /// Enable edns, for larger records
     pub edns0: bool,
-    /// Use DNSSec to validate the request
+    /// Use DNSSEC to validate the request
     pub validate: bool,
     /// The ip_strategy for the Resolver to use when lookup Ipv4 or Ipv6 addresses
     pub ip_strategy: LookupIpStrategy,
@@ -781,10 +907,20 @@ pub struct ResolverOpts {
     /// Where more than one nameserver is configured, this configures the resolver to send queries
     /// to a number of servers in parallel. Defaults to 2; 0 or 1 will execute requests serially.
     pub num_concurrent_reqs: usize,
-    /// Preserve all intermediate records in the lookup response, suchas CNAME records
+    /// Preserve all intermediate records in the lookup response, such as CNAME records
     pub preserve_intermediates: bool,
     /// Try queries over TCP if they fail over UDP.
     pub try_tcp_on_error: bool,
+    /// The server ordering strategy that the resolver should use.
+    pub server_ordering_strategy: ServerOrderingStrategy,
+    /// Request upstream recursive resolvers to not perform any recursion.
+    ///
+    /// This is true by default, disabling this is useful for requesting single records, but may prevent successful resolution.
+    pub recursion_desired: bool,
+    /// This is true by default, disabling this is useful for requesting single records, but may prevent successful resolution.
+    pub authentic_data: bool,
+    /// Shuffle DNS servers before each query.
+    pub shuffle_dns_servers: bool,
 }
 
 impl Default for ResolverOpts {
@@ -813,6 +949,10 @@ impl Default for ResolverOpts {
             preserve_intermediates: true,
 
             try_tcp_on_error: false,
+            server_ordering_strategy: ServerOrderingStrategy::default(),
+            recursion_desired: true,
+            authentic_data: false,
+            shuffle_dns_servers: false,
         }
     }
 }
