@@ -7,7 +7,7 @@
 
 //! An LRU cache designed for work with DNS lookups
 
-use std::convert::TryFrom;
+use std::collections::HashMap;
 use std::sync::Arc;
 use std::time::{Duration, Instant};
 
@@ -42,10 +42,37 @@ impl LruValue {
     fn ttl(&self, now: Instant) -> Duration {
         self.valid_until.saturating_duration_since(now)
     }
+
+    fn with_updated_ttl(&self, now: Instant) -> Self {
+        let lookup = match self.lookup {
+            Ok(ref lookup) => {
+                let records = lookup
+                    .records()
+                    .iter()
+                    .map(|record| {
+                        let mut record = record.clone();
+                        record.set_ttl(self.ttl(now).as_secs() as u32);
+                        record
+                    })
+                    .collect::<Vec<Record>>();
+                Ok(Lookup::new_with_deadline(
+                    lookup.query().clone(),
+                    Arc::from(records),
+                    self.valid_until,
+                ))
+            }
+            Err(ref e) => Err(e.clone()),
+        };
+        Self {
+            lookup,
+            valid_until: self.valid_until,
+        }
+    }
 }
 
+/// And LRU eviction cache specifically for storing DNS records
 #[derive(Clone, Debug)]
-pub(crate) struct DnsLru {
+pub struct DnsLru {
     cache: Arc<Mutex<LruCache<Query, LruValue>>>,
     /// A minimum TTL value for positive responses.
     ///
@@ -93,7 +120,7 @@ pub(crate) struct DnsLru {
 ///   shouldn't cause any issue as this will never be used in serialization,
 ///   but understand that this would be outside the standard range.
 #[derive(Copy, Clone, Debug, Default)]
-pub(crate) struct TtlConfig {
+pub struct TtlConfig {
     /// An optional minimum TTL value for positive responses.
     ///
     /// Positive responses with TTLs under `positive_min_ttl` will use
@@ -117,7 +144,8 @@ pub(crate) struct TtlConfig {
 }
 
 impl TtlConfig {
-    pub(crate) fn from_opts(opts: &config::ResolverOpts) -> Self {
+    /// Construct the LRU based on the ResolverOpts configuration
+    pub fn from_opts(opts: &config::ResolverOpts) -> Self {
         Self {
             positive_min_ttl: opts.positive_min_ttl,
             negative_min_ttl: opts.negative_min_ttl,
@@ -128,7 +156,13 @@ impl TtlConfig {
 }
 
 impl DnsLru {
-    pub(crate) fn new(capacity: usize, ttl_cfg: TtlConfig) -> Self {
+    /// Construct a new cache
+    ///
+    /// # Arguments
+    ///
+    /// * `capacity` - size in number of records, this can be the max size of 2048 (record size) * `capacity`
+    /// * `ttl_cfg` - force minimums and maximums for cached records
+    pub fn new(capacity: usize, ttl_cfg: TtlConfig) -> Self {
         let TtlConfig {
             positive_min_ttl,
             negative_min_ttl,
@@ -187,6 +221,54 @@ impl DnsLru {
         lookup
     }
 
+    /// inserts a record based on the name and type.
+    ///
+    /// # Arguments
+    ///
+    /// * `original_query` - is used for matching the records that should be returned
+    /// * `records` - the records will be partitioned by type and name for storage in the cache
+    /// * `now` - current time for use in associating TTLs
+    ///
+    /// # Return
+    ///
+    /// This should always return some records, but will be None if there are no records or the original_query matches none
+    pub fn insert_records(
+        &self,
+        original_query: Query,
+        records: impl Iterator<Item = Record>,
+        now: Instant,
+    ) -> Option<Lookup> {
+        // collect all records by name
+        let records = records.fold(
+            HashMap::<Query, Vec<(Record, u32)>>::new(),
+            |mut map, record| {
+                let mut query = Query::query(record.name().clone(), record.record_type());
+                query.set_query_class(record.dns_class());
+
+                let ttl = record.ttl();
+
+                map.entry(query)
+                    .or_insert_with(Vec::default)
+                    .push((record, ttl));
+
+                map
+            },
+        );
+
+        // now insert by record type and name
+        let mut lookup = None;
+        for (query, records_and_ttl) in records {
+            let is_query = original_query == query;
+            let inserted = self.insert(query, records_and_ttl, now);
+
+            if is_query {
+                lookup = Some(inserted)
+            }
+        }
+
+        lookup
+    }
+
     /// Generally for inserting a set of records that have already been cached, but with a different Query.
     pub(crate) fn duplicate(&self, query: Query, lookup: Lookup, ttl: u32, now: Instant) -> Lookup {
         let ttl = Duration::from_secs(u64::from(ttl));
@@ -260,15 +342,14 @@ impl DnsLru {
         error
     }
 
-    /// This needs to be mut b/c it's an LRU, meaning the ordering of elements will potentially change on retrieval...
-    pub(crate) fn get(&self, query: &Query, now: Instant) -> Option<Result<Lookup, ResolveError>> {
+    /// Based on the query, see if there are any records available
+    pub fn get(&self, query: &Query, now: Instant) -> Option<Result<Lookup, ResolveError>> {
         let mut out_of_date = false;
         let mut cache = self.cache.lock();
         let lookup = cache.get_mut(query).and_then(|value| {
             if value.is_current(now) {
                 out_of_date = false;
-                let mut result = value.lookup.clone();
-
+                let mut result = value.with_updated_ttl(now).lookup;
                 if let Err(ref mut err) = result {
                     Self::nx_error_with_ttl(err, value.ttl(now));
                 }
@@ -293,11 +374,11 @@ impl DnsLru {
 // see also the lookup_tests.rs in integration-tests crate
 #[cfg(test)]
 mod tests {
-    use std::net::*;
     use std::str::FromStr;
     use std::time::*;
 
     use proto::op::{Query, ResponseCode};
+    use proto::rr::rdata::A;
     use proto::rr::{Name, RData, RecordType};
 
     use super::*;
@@ -328,10 +409,10 @@ mod tests {
         let query = Query::query(name.clone(), RecordType::A);
         // record should have TTL of 1 second.
         let ips_ttl = vec![(
-            Record::from_rdata(name.clone(), 1, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+            Record::from_rdata(name.clone(), 1, RData::A(A::new(127, 0, 0, 1))),
             1,
         )];
-        let ips = vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))];
+        let ips = vec![RData::A(A::new(127, 0, 0, 1))];
 
         // configure the cache with a minimum TTL of 2 seconds.
         let ttls = TtlConfig {
@@ -348,7 +429,7 @@ mod tests {
 
         // record should have TTL of 3 seconds.
         let ips_ttl = vec![(
-            Record::from_rdata(name, 3, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+            Record::from_rdata(name, 3, RData::A(A::new(127, 0, 0, 1))),
             3,
         )];
 
@@ -418,10 +499,10 @@ mod tests {
         let query = Query::query(name.clone(), RecordType::A);
         // record should have TTL of 62 seconds.
         let ips_ttl = vec![(
-            Record::from_rdata(name.clone(), 62, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+            Record::from_rdata(name.clone(), 62, RData::A(A::new(127, 0, 0, 1))),
             62,
         )];
-        let ips = vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))];
+        let ips = vec![RData::A(A::new(127, 0, 0, 1))];
 
         // configure the cache with a maximum TTL of 60 seconds.
         let ttls = TtlConfig {
@@ -438,7 +519,7 @@ mod tests {
 
         // record should have TTL of 59 seconds.
         let ips_ttl = vec![(
-            Record::from_rdata(name, 59, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+            Record::from_rdata(name, 59, RData::A(A::new(127, 0, 0, 1))),
             59,
         )];
 
@@ -507,10 +588,10 @@ mod tests {
         let name = Name::from_str("www.example.com.").unwrap();
         let query = Query::query(name.clone(), RecordType::A);
         let ips_ttl = vec![(
-            Record::from_rdata(name, 1, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+            Record::from_rdata(name, 1, RData::A(A::new(127, 0, 0, 1))),
             1,
         )];
-        let ips = vec![RData::A(Ipv4Addr::new(127, 0, 0, 1))];
+        let ips = vec![RData::A(A::new(127, 0, 0, 1))];
         let lru = DnsLru::new(1, TtlConfig::default());
 
         let rc_ips = lru.insert(query.clone(), ips_ttl, now);
@@ -520,6 +601,33 @@ mod tests {
         assert_eq!(*rc_ips.iter().next().unwrap(), ips[0]);
     }
 
+    #[test]
+    fn test_update_ttl() {
+        let now = Instant::now();
+
+        let name = Name::from_str("www.example.com.").unwrap();
+        let query = Query::query(name.clone(), RecordType::A);
+        let ips_ttl = vec![(
+            Record::from_rdata(name, 10, RData::A(A::new(127, 0, 0, 1))),
+            10,
+        )];
+        let ips = vec![RData::A(A::new(127, 0, 0, 1))];
+        let lru = DnsLru::new(1, TtlConfig::default());
+
+        let rc_ips = lru.insert(query.clone(), ips_ttl, now);
+        assert_eq!(*rc_ips.iter().next().unwrap(), ips[0]);
+
+        let ttl = lru
+            .get(&query, now + Duration::from_secs(2))
+            .unwrap()
+            .expect("records should exist")
+            .record_iter()
+            .next()
+            .unwrap()
+            .ttl();
+        assert!(ttl <= 8);
+    }
+
     #[test]
     fn test_insert_ttl() {
         let now = Instant::now();
@@ -528,17 +636,17 @@ mod tests {
         // TTL should be 1
         let ips_ttl = vec![
             (
-                Record::from_rdata(name.clone(), 1, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+                Record::from_rdata(name.clone(), 1, RData::A(A::new(127, 0, 0, 1))),
                 1,
             ),
             (
-                Record::from_rdata(name, 2, RData::A(Ipv4Addr::new(127, 0, 0, 2))),
+                Record::from_rdata(name, 2, RData::A(A::new(127, 0, 0, 2))),
                 2,
             ),
         ];
         let ips = vec![
-            RData::A(Ipv4Addr::new(127, 0, 0, 1)),
-            RData::A(Ipv4Addr::new(127, 0, 0, 2)),
+            RData::A(A::new(127, 0, 0, 1)),
+            RData::A(A::new(127, 0, 0, 2)),
         ];
         let lru = DnsLru::new(1, TtlConfig::default());
 
@@ -564,17 +672,17 @@ mod tests {
         // TTL should be 1
         let ips_ttl = vec![
             (
-                Record::from_rdata(name.clone(), 1, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+                Record::from_rdata(name.clone(), 1, RData::A(A::new(127, 0, 0, 1))),
                 1,
             ),
             (
-                Record::from_rdata(name, 2, RData::A(Ipv4Addr::new(127, 0, 0, 2))),
+                Record::from_rdata(name, 2, RData::A(A::new(127, 0, 0, 2))),
                 2,
             ),
         ];
         let ips = vec![
-            RData::A(Ipv4Addr::new(127, 0, 0, 1)),
-            RData::A(Ipv4Addr::new(127, 0, 0, 2)),
+            RData::A(A::new(127, 0, 0, 1)),
+            RData::A(A::new(127, 0, 0, 2)),
         ];
 
         // this cache should override the TTL of 1 seconds with the configured
@@ -624,17 +732,17 @@ mod tests {
         // TTL should be 500
         let ips_ttl = vec![
             (
-                Record::from_rdata(name.clone(), 400, RData::A(Ipv4Addr::new(127, 0, 0, 1))),
+                Record::from_rdata(name.clone(), 400, RData::A(A::new(127, 0, 0, 1))),
                 400,
             ),
             (
-                Record::from_rdata(name, 500, RData::A(Ipv4Addr::new(127, 0, 0, 2))),
+                Record::from_rdata(name, 500, RData::A(A::new(127, 0, 0, 2))),
                 500,
             ),
         ];
         let ips = vec![
-            RData::A(Ipv4Addr::new(127, 0, 0, 1)),
-            RData::A(Ipv4Addr::new(127, 0, 0, 2)),
+            RData::A(A::new(127, 0, 0, 1)),
+            RData::A(A::new(127, 0, 0, 2)),
         ];
 
         // this cache should override the TTL of 500 seconds with the configured
