@@ -1,4 +1,4 @@
-// Copyright 2015-2020 Benjamin Fry <benjaminfry@me.com>
+// Copyright 2015-2023 Benjamin Fry <benjaminfry@me.com>
 //
 // Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
 // http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
@@ -7,16 +7,21 @@
 
 //! Error types for the crate
 
-use std::cmp::Ordering;
-use std::{fmt, io, sync};
+use std::{cmp::Ordering, fmt, io, sync};
 
 use thiserror::Error;
+use tracing::debug;
+
+use crate::proto::{
+    error::{ProtoError, ProtoErrorKind},
+    op::{Query, ResponseCode},
+    rr::{
+        rdata::SOA,
+        resource::{Record, RecordRef},
+    },
+    xfer::{retry_dns_handle::RetryableError, DnsResponse},
+};
 
-use crate::proto::error::{ProtoError, ProtoErrorKind};
-use crate::proto::op::{Query, ResponseCode};
-use crate::proto::rr::rdata::SOA;
-use crate::proto::xfer::retry_dns_handle::RetryableError;
-use crate::proto::xfer::DnsResponse;
 #[cfg(feature = "backtrace")]
 use crate::proto::{trace, ExtBacktrace};
 
@@ -41,12 +46,12 @@ pub enum ResolveErrorKind {
     NoConnections,
 
     /// No records were found for a query
-    #[error("no record found for {query}")]
+    #[error("no record found for {:?}", query)]
     NoRecordsFound {
         /// The query for which no records were found.
         query: Box<Query>,
-        /// If an SOA is present, then this is an authoritative response.
-        soa: Option<Box<SOA>>,
+        /// If an SOA is present, then this is an authoritative response or a referral to another nameserver, see the negative_type field.
+        soa: Option<Box<Record<SOA>>>,
         /// negative ttl, as determined from DnsResponse::negative_ttl
         ///  this will only be present if the SOA was also present.
         negative_ttl: Option<u32>,
@@ -110,7 +115,7 @@ pub struct ResolveError {
 impl ResolveError {
     pub(crate) fn nx_error(
         query: Query,
-        soa: Option<SOA>,
+        soa: Option<Record<SOA>>,
         negative_ttl: Option<u32>,
         response_code: ResponseCode,
         trusted: bool,
@@ -144,7 +149,7 @@ impl ResolveError {
 
     /// A conversion to determine if the response is an error
     pub fn from_response(response: DnsResponse, trust_nx: bool) -> Result<DnsResponse, Self> {
-        debug!("Response:{}", response.header());
+        debug!("Response:{}", *response);
 
         match response.response_code() {
             response_code @ ResponseCode::ServFail
@@ -165,9 +170,9 @@ impl ResolveError {
             | response_code @ ResponseCode::BADALG
             | response_code @ ResponseCode::BADTRUNC
             | response_code @ ResponseCode::BADCOOKIE => {
-                let mut response = response;
-                let soa = response.soa();
-                let query = response.take_queries().drain(..).next().unwrap_or_default();
+                let response = response;
+                let soa = response.soa().as_ref().map(RecordRef::to_owned);
+                let query = response.queries().iter().next().cloned().unwrap_or_default();
                 let error_kind = ResolveErrorKind::NoRecordsFound {
                     query: Box::new(query),
                     soa: soa.map(Box::new),
@@ -184,13 +189,17 @@ impl ResolveError {
             response_code @ ResponseCode::NoError
             if !response.contains_answer() && !response.truncated() => {
                 // TODO: if authoritative, this is cacheable, store a TTL (currently that requires time, need a "now" here)
-                // let valid_until = if response.is_authoritative() { now + response.get_negative_ttl() };
+                // let valid_until = if response.authoritative() { now + response.negative_ttl() };
 
-                let mut response = response;
-                let soa = response.soa();
+                let  response = response;
+                let soa = response.soa().as_ref().map(RecordRef::to_owned);
                 let negative_ttl = response.negative_ttl();
-                let trusted = if response_code == ResponseCode::NoError { false } else { trust_nx };
-                let query = response.take_queries().drain(..).next().unwrap_or_default();
+                // Note: improperly configured servers may do recursive lookups and return bad SOA
+                // records here via AS112 (blackhole-1.iana.org. etc)
+                // Such servers should be marked not trusted, as they may break reverse lookups
+                // for local hosts.
+                let trusted = trust_nx && soa.is_some();
+                let query = response.into_message().take_queries().drain(..).next().unwrap_or_default();
                 let error_kind = ResolveErrorKind::NoRecordsFound {
                     query: Box::new(query),
                     soa: soa.map(Box::new),
@@ -346,6 +355,6 @@ impl From<ResolveError> for io::Error {
 
 impl<T> From<sync::PoisonError<T>> for ResolveError {
     fn from(e: sync::PoisonError<T>) -> Self {
-        ResolveErrorKind::Msg(format!("lock was poisoned, this is non-recoverable: {}", e)).into()
+        ResolveErrorKind::Msg(format!("lock was poisoned, this is non-recoverable: {e}")).into()
     }
 }
