@@ -1,23 +1,32 @@
+// Copyright 2015-2022 Benjamin Fry <benjaminfry@me.com>
+//
+// Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
+// http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
+// http://opensource.org/licenses/MIT>, at your option. This file may not be
+// copied, modified, or distributed except according to those terms.
+
+use std::future::Future;
 use std::net::SocketAddr;
 
-use crate::name_server::RuntimeProvider;
 use crate::tls::CLIENT_CONFIG;
 
 use proto::https::{HttpsClientConnect, HttpsClientStream, HttpsClientStreamBuilder};
+use proto::tcp::{Connect, DnsTcpStream};
 use proto::xfer::{DnsExchange, DnsExchangeConnect};
 use proto::TokioTime;
 
 use crate::config::TlsClientConfig;
 
 #[allow(clippy::type_complexity)]
-pub(crate) fn new_https_stream<R>(
+#[allow(unused)]
+pub(crate) fn new_https_stream<S>(
     socket_addr: SocketAddr,
     bind_addr: Option<SocketAddr>,
     dns_name: String,
     client_config: Option<TlsClientConfig>,
-) -> DnsExchangeConnect<HttpsClientConnect<R::Tcp>, HttpsClientStream, TokioTime>
+) -> DnsExchangeConnect<HttpsClientConnect<S>, HttpsClientStream, TokioTime>
 where
-    R: RuntimeProvider,
+    S: Connect,
 {
     let client_config = client_config.map_or_else(
         || CLIENT_CONFIG.clone(),
@@ -28,7 +37,31 @@ where
     if let Some(bind_addr) = bind_addr {
         https_builder.bind_addr(bind_addr);
     }
-    DnsExchange::connect(https_builder.build::<R::Tcp>(socket_addr, dns_name))
+    DnsExchange::connect(https_builder.build::<S>(socket_addr, dns_name))
+}
+
+#[allow(clippy::type_complexity)]
+pub(crate) fn new_https_stream_with_future<S, F>(
+    future: F,
+    socket_addr: SocketAddr,
+    dns_name: String,
+    client_config: Option<TlsClientConfig>,
+) -> DnsExchangeConnect<HttpsClientConnect<S>, HttpsClientStream, TokioTime>
+where
+    S: DnsTcpStream,
+    F: Future<Output = std::io::Result<S>> + Send + Unpin + 'static,
+{
+    let client_config = client_config.map_or_else(
+        || CLIENT_CONFIG.clone(),
+        |TlsClientConfig(client_config)| client_config,
+    );
+
+    DnsExchange::connect(HttpsClientStreamBuilder::build_with_future(
+        future,
+        client_config,
+        socket_addr,
+        dns_name,
+    ))
 }
 
 #[cfg(test)]
@@ -38,7 +71,8 @@ mod tests {
     use tokio::runtime::Runtime;
 
     use crate::config::{ResolverConfig, ResolverOpts};
-    use crate::{TokioAsyncResolver, TokioHandle};
+    use crate::name_server::TokioConnectionProvider;
+    use crate::TokioAsyncResolver;
 
     fn https_test(config: ResolverConfig) {
         let io_loop = Runtime::new().unwrap();
@@ -49,9 +83,8 @@ mod tests {
                 try_tcp_on_error: true,
                 ..ResolverOpts::default()
             },
-            TokioHandle,
-        )
-        .expect("failed to create resolver");
+            TokioConnectionProvider::default(),
+        );
 
         let response = io_loop
             .block_on(resolver.lookup_ip("www.example.com."))
@@ -91,6 +124,11 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_google_https() {
+        https_test(ResolverConfig::google_https())
+    }
+
     #[test]
     fn test_cloudflare_https() {
         https_test(ResolverConfig::cloudflare_https())
