@@ -17,55 +17,60 @@ use smallvec::SmallVec;
 
 use proto::xfer::{DnsHandle, DnsRequest, DnsResponse, FirstAnswer};
 use proto::Time;
+use tracing::debug;
 
-use crate::config::{ResolverConfig, ResolverOpts};
+use rand::thread_rng as rng;
+use rand::Rng;
+
+use crate::config::{NameServerConfigGroup, ResolverConfig, ResolverOpts, ServerOrderingStrategy};
 use crate::error::{ResolveError, ResolveErrorKind};
 #[cfg(feature = "mdns")]
 use crate::name_server;
-use crate::name_server::{ConnectionProvider, NameServer};
+use crate::name_server::connection_provider::{ConnectionProvider, GenericConnector};
+use crate::name_server::name_server::NameServer;
+use crate::name_server::RuntimeProvider;
 #[cfg(test)]
 #[cfg(feature = "tokio-runtime")]
-use crate::name_server::{TokioConnection, TokioConnectionProvider, TokioHandle};
+use crate::name_server::TokioRuntimeProvider;
+
+/// Abstract interface for mocking purpose
+#[derive(Clone)]
+pub struct NameServerPool<P: ConnectionProvider + Send + 'static> {
+    // TODO: switch to FuturesMutex (Mutex will have some undesirable locking)
+    datagram_conns: Arc<[NameServer<P>]>, /* All NameServers must be the same type */
+    stream_conns: Arc<[NameServer<P>]>,   /* All NameServers must be the same type */
+    #[cfg(feature = "mdns")]
+    mdns_conns: NameServer<P>, /* All NameServers must be the same type */
+    options: ResolverOpts,
+}
 
 /// A pool of NameServers
 ///
 /// This is not expected to be used directly, see [crate::AsyncResolver].
-#[derive(Clone)]
-pub struct NameServerPool<
-    C: DnsHandle<Error = ResolveError> + Send + Sync + 'static,
-    P: ConnectionProvider<Conn = C> + Send + 'static,
-> {
-    // TODO: switch to FuturesMutex (Mutex will have some undesireable locking)
-    datagram_conns: Arc<[NameServer<C, P>]>, /* All NameServers must be the same type */
-    stream_conns: Arc<[NameServer<C, P>]>,   /* All NameServers must be the same type */
-    #[cfg(feature = "mdns")]
-    mdns_conns: NameServer<C, P>, /* All NameServers must be the same type */
-    options: ResolverOpts,
-}
+pub type GenericNameServerPool<P> = NameServerPool<GenericConnector<P>>;
 
 #[cfg(test)]
 #[cfg(feature = "tokio-runtime")]
-impl NameServerPool<TokioConnection, TokioConnectionProvider> {
-    pub(crate) fn from_config(
+impl GenericNameServerPool<TokioRuntimeProvider> {
+    pub(crate) fn tokio_from_config(
         config: &ResolverConfig,
         options: &ResolverOpts,
-        runtime: TokioHandle,
+        runtime: TokioRuntimeProvider,
     ) -> Self {
-        Self::from_config_with_provider(config, options, TokioConnectionProvider::new(runtime))
+        Self::from_config_with_provider(config, options, GenericConnector::new(runtime))
     }
 }
 
-impl<C, P> NameServerPool<C, P>
+impl<P> NameServerPool<P>
 where
-    C: DnsHandle<Error = ResolveError> + Sync + 'static,
-    P: ConnectionProvider<Conn = C> + 'static,
+    P: ConnectionProvider + 'static,
 {
     pub(crate) fn from_config_with_provider(
         config: &ResolverConfig,
         options: &ResolverOpts,
         conn_provider: P,
     ) -> Self {
-        let datagram_conns: Vec<NameServer<C, P>> = config
+        let datagram_conns: Vec<NameServer<P>> = config
             .name_servers()
             .iter()
             .filter(|ns_config| ns_config.protocol.is_datagram())
@@ -79,11 +84,11 @@ where
                 #[cfg(not(feature = "dns-over-rustls"))]
                 let ns_config = { ns_config.clone() };
 
-                NameServer::<C, P>::new_with_provider(ns_config, *options, conn_provider.clone())
+                NameServer::new(ns_config, *options, conn_provider.clone())
             })
             .collect();
 
-        let stream_conns: Vec<NameServer<C, P>> = config
+        let stream_conns: Vec<NameServer<P>> = config
             .name_servers()
             .iter()
             .filter(|ns_config| ns_config.protocol.is_stream())
@@ -97,7 +102,7 @@ where
                 #[cfg(not(feature = "dns-over-rustls"))]
                 let ns_config = { ns_config.clone() };
 
-                NameServer::<C, P>::new_with_provider(ns_config, *options, conn_provider.clone())
+                NameServer::new(ns_config, *options, conn_provider.clone())
             })
             .collect();
 
@@ -110,12 +115,38 @@ where
         }
     }
 
+    /// Construct a NameServerPool from a set of name server configs
+    pub fn from_config(
+        name_servers: NameServerConfigGroup,
+        options: &ResolverOpts,
+        conn_provider: P,
+    ) -> Self {
+        let map_config_to_ns =
+            |ns_config| NameServer::new(ns_config, *options, conn_provider.clone());
+
+        let (datagram, stream): (Vec<_>, Vec<_>) = name_servers
+            .into_inner()
+            .into_iter()
+            .partition(|ns| ns.protocol.is_datagram());
+
+        let datagram_conns: Vec<_> = datagram.into_iter().map(map_config_to_ns).collect();
+        let stream_conns: Vec<_> = stream.into_iter().map(map_config_to_ns).collect();
+
+        Self {
+            datagram_conns: Arc::from(datagram_conns),
+            stream_conns: Arc::from(stream_conns),
+            #[cfg(feature = "mdns")]
+            mdns_conns: name_server::mdns_nameserver(*options, conn_provider.clone(), false),
+            options: *options,
+        }
+    }
+
     #[doc(hidden)]
     #[cfg(not(feature = "mdns"))]
     pub fn from_nameservers(
         options: &ResolverOpts,
-        datagram_conns: Vec<NameServer<C, P>>,
-        stream_conns: Vec<NameServer<C, P>>,
+        datagram_conns: Vec<NameServer<P>>,
+        stream_conns: Vec<NameServer<P>>,
     ) -> Self {
         Self {
             datagram_conns: Arc::from(datagram_conns),
@@ -128,11 +159,11 @@ where
     #[cfg(feature = "mdns")]
     pub fn from_nameservers(
         options: &ResolverOpts,
-        datagram_conns: Vec<NameServer<C, P>>,
-        stream_conns: Vec<NameServer<C, P>>,
-        mdns_conns: NameServer<C, P>,
+        datagram_conns: Vec<NameServer<P>>,
+        stream_conns: Vec<NameServer<P>>,
+        mdns_conns: NameServer<P>,
     ) -> Self {
-        NameServerPool {
+        GenericNameServerPool {
             datagram_conns: Arc::from(datagram_conns),
             stream_conns: Arc::from(stream_conns),
             mdns_conns,
@@ -145,8 +176,8 @@ where
     #[allow(dead_code)]
     fn from_nameservers_test(
         options: &ResolverOpts,
-        datagram_conns: Arc<[NameServer<C, P>]>,
-        stream_conns: Arc<[NameServer<C, P>]>,
+        datagram_conns: Arc<[NameServer<P>]>,
+        stream_conns: Arc<[NameServer<P>]>,
     ) -> Self {
         Self {
             datagram_conns,
@@ -159,40 +190,41 @@ where
     #[cfg(feature = "mdns")]
     fn from_nameservers_test(
         options: &ResolverOpts,
-        datagram_conns: Arc<[NameServer<C, P>]>,
-        stream_conns: Arc<[NameServer<C, P>]>,
-        mdns_conns: NameServer<C, P>,
+        datagram_conns: Arc<[NameServer<P>]>,
+        stream_conns: Arc<[NameServer<P>]>,
+        mdns_conns: NameServer<P>,
     ) -> Self {
-        NameServerPool {
+        GenericNameServerPool {
             datagram_conns,
             stream_conns,
             mdns_conns,
             options: *options,
-            conn_provider,
         }
     }
 
     async fn try_send(
         opts: ResolverOpts,
-        conns: Arc<[NameServer<C, P>]>,
+        conns: Arc<[NameServer<P>]>,
         request: DnsRequest,
     ) -> Result<DnsResponse, ResolveError> {
-        let mut conns: Vec<NameServer<C, P>> = conns.to_vec();
+        let mut conns: Vec<NameServer<P>> = conns.to_vec();
 
-        // select the highest priority connection
-        //   reorder the connections based on current view...
-        //   this reorders the inner set
-        conns.sort_unstable();
+        match opts.server_ordering_strategy {
+            // select the highest priority connection
+            //   reorder the connections based on current view...
+            //   this reorders the inner set
+            ServerOrderingStrategy::QueryStatistics => conns.sort_unstable(),
+            ServerOrderingStrategy::UserProvidedOrder => {}
+        }
         let request_loop = request.clone();
 
         parallel_conn_loop(conns, request_loop, opts).await
     }
 }
 
-impl<C, P> DnsHandle for NameServerPool<C, P>
+impl<P> DnsHandle for NameServerPool<P>
 where
-    C: DnsHandle<Error = ResolveError> + Sync + 'static,
-    P: ConnectionProvider<Conn = C> + 'static,
+    P: ConnectionProvider + 'static,
 {
     type Response = Pin<Box<dyn Stream<Item = Result<DnsResponse, ResolveError>> + Send>>;
     type Error = ResolveError;
@@ -232,7 +264,7 @@ where
                     Ok(response)
                 }
                 Err(e) if opts.try_tcp_on_error || e.is_no_connections() => {
-                    debug!("error received, retrying over TCP");
+                    debug!("error from UDP, retrying over TCP: {}", e);
                     Err(e)
                 }
                 result => return result,
@@ -268,14 +300,13 @@ where
 
 // TODO: we should be able to have a self-referential future here with Pin and not require cloned conns
 /// An async function that will loop over all the conns with a max parallel request count of ops.num_concurrent_req
-async fn parallel_conn_loop<C, P>(
-    mut conns: Vec<NameServer<C, P>>,
+async fn parallel_conn_loop<P>(
+    mut conns: Vec<NameServer<P>>,
     request: DnsRequest,
     opts: ResolverOpts,
 ) -> Result<DnsResponse, ResolveError>
 where
-    C: DnsHandle<Error = ResolveError> + 'static,
-    P: ConnectionProvider<Conn = C> + 'static,
+    P: ConnectionProvider + 'static,
 {
     let mut err = ResolveError::no_connections();
     // If the name server we're trying is giving us backpressure by returning ProtoErrorKind::Busy,
@@ -289,21 +320,37 @@ where
     // close to the connection, which means the top level resolution might take substantially longer
     // to fire than the timeout configured in `ResolverOpts`.
     let mut backoff = Duration::from_millis(20);
-    let mut busy = SmallVec::<[NameServer<C, P>; 2]>::new();
+    let mut busy = SmallVec::<[NameServer<P>; 2]>::new();
 
     loop {
         let request_cont = request.clone();
 
         // construct the parallel requests, 2 is the default
-        let mut par_conns = SmallVec::<[NameServer<C, P>; 2]>::new();
+        let mut par_conns = SmallVec::<[NameServer<P>; 2]>::new();
         let count = conns.len().min(opts.num_concurrent_reqs.max(1));
-        for conn in conns.drain(..count) {
-            par_conns.push(conn);
+
+        // Shuffe DNS NameServers to avoid overloads to the first configured ones
+        if opts.shuffle_dns_servers {
+            for _ in 0..count {
+                let idx = rng().gen_range(0..conns.len());
+
+                // UNWRAP: swap_remove has an implicit panicking bounds check. This should
+                // never fail because we check that conns is not empty and generate the idx
+                // to explicitly be in range.
+                par_conns.push(conns.swap_remove(idx));
+            }
+        } else {
+            for conn in conns.drain(..count) {
+                par_conns.push(conn);
+            }
         }
 
         if par_conns.is_empty() {
             if !busy.is_empty() && backoff < Duration::from_millis(300) {
-                P::Time::delay_for(backoff).await;
+                <<P as ConnectionProvider>::RuntimeProvider as RuntimeProvider>::Timer::delay_for(
+                    backoff,
+                )
+                .await;
                 conns.extend(busy.drain(..));
                 backoff *= 2;
                 continue;
@@ -371,6 +418,7 @@ mod mdns {
     }
 }
 
+#[allow(clippy::large_enum_variant)]
 pub(crate) enum Local {
     #[allow(dead_code)]
     ResolveStream(Pin<Box<dyn Stream<Item = Result<DnsResponse, ResolveError>> + Send>>),
@@ -379,7 +427,7 @@ pub(crate) enum Local {
 
 impl Local {
     fn is_local(&self) -> bool {
-        matches!(*self, Local::ResolveStream(..))
+        matches!(*self, Self::ResolveStream(..))
     }
 
     /// Takes the stream
@@ -389,7 +437,7 @@ impl Local {
     /// Panics if this is in fact a Local::NotMdns
     fn take_stream(self) -> Pin<Box<dyn Stream<Item = Result<DnsResponse, ResolveError>> + Send>> {
         match self {
-            Local::ResolveStream(future) => future,
+            Self::ResolveStream(future) => future,
             _ => panic!("non Local queries have no future, see take_message()"),
         }
     }
@@ -401,7 +449,7 @@ impl Local {
     /// Panics if this is in fact a Local::ResolveStream
     fn take_request(self) -> DnsRequest {
         match self {
-            Local::NotMdns(request) => request,
+            Self::NotMdns(request) => request,
             _ => panic!("Local queries must be polled, see take_future()"),
         }
     }
@@ -412,9 +460,9 @@ impl Stream for Local {
 
     fn poll_next(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
         match *self {
-            Local::ResolveStream(ref mut ns) => ns.as_mut().poll_next(cx),
+            Self::ResolveStream(ref mut ns) => ns.as_mut().poll_next(cx),
             // TODO: making this a panic for now
-            Local::NotMdns(..) => panic!("Local queries that are not mDNS should not be polled"), //Local::NotMdns(message) => return Err(ResolveErrorKind::Message("not mDNS")),
+            Self::NotMdns(..) => panic!("Local queries that are not mDNS should not be polled"), //Local::NotMdns(message) => return Err(ResolveErrorKind::Message("not mDNS")),
         }
     }
 }
@@ -435,16 +483,19 @@ mod tests {
     use super::*;
     use crate::config::NameServerConfig;
     use crate::config::Protocol;
+    use crate::name_server::TokioRuntimeProvider;
+    use crate::name_server::{GenericNameServer, TokioConnectionProvider};
 
     #[ignore]
     // because of there is a real connection that needs a reasonable timeout
     #[test]
+    #[allow(clippy::uninlined_format_args)]
     fn test_failed_then_success_pool() {
         let config1 = NameServerConfig {
             socket_addr: SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 252)), 253),
             protocol: Protocol::Udp,
             tls_dns_name: None,
-            trust_nx_responses: false,
+            trust_negative_responses: false,
             #[cfg(feature = "dns-over-rustls")]
             tls_config: None,
             bind_addr: None,
@@ -454,7 +505,7 @@ mod tests {
             socket_addr: SocketAddr::new(IpAddr::V4(Ipv4Addr::new(8, 8, 8, 8)), 53),
             protocol: Protocol::Udp,
             tls_dns_name: None,
-            trust_nx_responses: false,
+            trust_negative_responses: false,
             #[cfg(feature = "dns-over-rustls")]
             tls_config: None,
             bind_addr: None,
@@ -465,10 +516,10 @@ mod tests {
         resolver_config.add_name_server(config2);
 
         let io_loop = Runtime::new().unwrap();
-        let mut pool = NameServerPool::<_, TokioConnectionProvider>::from_config(
+        let mut pool = GenericNameServerPool::tokio_from_config(
             &resolver_config,
             &ResolverOpts::default(),
-            TokioHandle,
+            TokioRuntimeProvider::new(),
         );
 
         let name = Name::parse("www.example.com.", None).unwrap();
@@ -509,16 +560,14 @@ mod tests {
 
     #[test]
     fn test_multi_use_conns() {
-        env_logger::try_init().ok();
-
         let io_loop = Runtime::new().unwrap();
-        let conn_provider = TokioConnectionProvider::new(TokioHandle);
+        let conn_provider = TokioConnectionProvider::default();
 
         let tcp = NameServerConfig {
             socket_addr: SocketAddr::new(IpAddr::V4(Ipv4Addr::new(8, 8, 8, 8)), 53),
             protocol: Protocol::Tcp,
             tls_dns_name: None,
-            trust_nx_responses: false,
+            trust_negative_responses: false,
             #[cfg(feature = "dns-over-rustls")]
             tls_config: None,
             bind_addr: None,
@@ -529,15 +578,21 @@ mod tests {
             ..ResolverOpts::default()
         };
         let ns_config = { tcp };
-        let name_server = NameServer::new_with_provider(ns_config, opts, conn_provider);
+        let name_server = GenericNameServer::new(ns_config, opts, conn_provider);
         let name_servers: Arc<[_]> = Arc::from([name_server]);
 
-        let mut pool = NameServerPool::from_nameservers_test(
+        #[cfg(not(feature = "mdns"))]
+        let mut pool = GenericNameServerPool::from_nameservers_test(
             &opts,
             Arc::from([]),
             Arc::clone(&name_servers),
-            #[cfg(feature = "mdns")]
-            name_server::mdns_nameserver(opts, TokioConnectionProvider::new(TokioHandle)),
+        );
+        #[cfg(feature = "mdns")]
+        let mut pool = GenericNameServerPool::from_nameservers_test(
+            &opts,
+            Arc::from([]),
+            Arc::clone(&name_servers),
+            name_server::mdns_nameserver(opts, TokioConnectionProvider::default(), false),
         );
 
         let name = Name::from_str("www.example.com.").unwrap();
@@ -558,7 +613,7 @@ mod tests {
                 .data()
                 .and_then(RData::as_a)
                 .expect("no a record available"),
-            Ipv4Addr::new(93, 184, 216, 34)
+            Ipv4Addr::new(93, 184, 216, 34).into()
         );
 
         assert!(
@@ -582,7 +637,7 @@ mod tests {
                 .data()
                 .and_then(RData::as_aaaa)
                 .expect("no aaaa record available"),
-            Ipv6Addr::new(0x2606, 0x2800, 0x0220, 0x0001, 0x0248, 0x1893, 0x25c8, 0x1946)
+            Ipv6Addr::new(0x2606, 0x2800, 0x0220, 0x0001, 0x0248, 0x1893, 0x25c8, 0x1946).into()
         );
 
         assert!(
